unit CommBtn; {Commented button}

interface

uses WinTypes, WinProcs, OWindows, ODialogs, MsgWin;

type
  PCommBtn=^TCommBtn;
  TCommBtn=object(TButton)
    TimerN:integer;
    Comment:string;
    MouseCaptured:boolean;
    commw:PWindowsObject;
    constructor Init (AParent:PWindowsObject; AnId:integer;
                      X,Y,W,H:integer; AComment:string);
    destructor Done; virtual;
    procedure WMMouseMove(var Msg:TMessage); virtual WM_FIRST+WM_MOUSEMOVE;
    procedure WMTimer(var Msg:TMessage); virtual WM_FIRST+WM_TIMER;
    procedure WMKillFocus(var Msg:TMessage); virtual WM_FIRST+WM_KILLFOCUS;
    procedure WMSetFocus(var Msg:TMessage); virtual WM_FIRST+WM_SETFOCUS;
    procedure WMLButtonDown (var Msg:TMessage); virtual wm_First+wm_LButtonDown;
    procedure ShowComment;
    procedure DeleteComment;
    procedure CheckAndInitTimer;
  end;

implementation

constructor TCommBtn.Init (AParent:PWindowsObject; AnId:integer;
                           X,Y,W,H:integer; AComment:string);
begin
  inherited Init (AParent,AnId,nil,X,Y,W,H,false);
  comment:=AComment;
  TimerN:=0; commw:=nil; MouseCaptured:=false;
end;

destructor TCommBtn.Done;
begin
  DeleteComment;
  inherited Done;
end;

procedure TCommBtn.WMKillFocus (var Msg:TMessage);
begin
  if (Msg.Receiver<>HWND(Msg.WParam)) then DeleteComment;
  DefWndProc(Msg);
end;

procedure TCommBtn.WMSetFocus (var Msg:TMessage);
begin
  if (TimerN>0) then begin KillTimer(HWindow,TimerN); TimerN:=0; end;
  DeleteComment;
  DefWndProc(Msg);
end;

procedure TCommBtn.WMLButtonDown (var Msg:TMessage);
begin
  DeleteComment;
  inherited WMLButtonDown(Msg);
end;

procedure TCommBtn.WMMouseMove (var Msg:TMessage);
var xpos,ypos:integer;
    rc:TRect;
begin
  if (TimerN>0) and (Msg.WParam<>0) then begin
    KillTimer(HWindow,TimerN); TimerN:=0;
    DefWndProc(Msg);
    Exit;
  end;
  if (MouseCaptured) then
    if (Msg.WParam<>0) then DeleteComment
    else begin
      xPos:=Msg.LParamLo; {horizontal position of cursor }
      yPos:=Msg.LParamHi; {vertical position of cursor   }
      GetClientRect(HWindow,rc);
      if (xPos<rc.left) or (xPos>rc.right) or (yPos<rc.top) or (yPos>rc.bottom) then
        DeleteComment;
    end
  else
    if (Msg.WParam=0) then CheckAndInitTimer;
  DefWndProc(Msg);
end;

procedure TCommBtn.CheckAndInitTimer;
begin
  if (commw<>nil) or (TimerN>0) then Exit;
  TimerN:=100; TimerN:=SetTimer(HWindow,TimerN,500,nil);
end;

procedure TCommBtn.ShowComment;
var p:TPoint;
    rc:TRect;
begin
  if (commw<>nil) then Exit;
  if (TimerN>0) then begin KillTimer(HWindow,TimerN); TimerN:=0; end;
  GetCursorPos(p);
  GetWindowRect(HWindow,rc);
  if (p.x<rc.left) or (p.x>rc.right) or (p.y<rc.top) or (p.y>rc.bottom) then exit;
  p.x:=rc.left; p.y:=rc.bottom+15;
  ScreenToClient(Parent^.Parent^.HWindow,p);
  commw:=StartMessage(Parent^.Parent,comment,p.x,p.y);
  SetCapture(HWindow);
  MouseCaptured:=true;
end;

procedure TCommBtn.DeleteComment;
begin
  if (TimerN>0) then begin KillTimer(HWindow,TimerN); TimerN:=0; end;
  if (commw<>nil) then begin FinishMessage(commw); commw:=nil; end;
  if (MouseCaptured) then begin ReleaseCapture; MouseCaptured:=false; end;
end;

procedure TCommBtn.WMTimer (var Msg:TMessage);
begin
  if (TimerN>0) then begin KillTimer(HWindow,TimerN); TimerN:=0; end;
  ShowComment;
end;

end.