{$A+,B-,D+,E+,F+,G-,I+,L+,N+,O+,P-,Q+,R+,S+,T-,V-,X+,Y+}
{$M 1024,0,0}

unit DynSys;

interface

type
  funcname=string[7];
const
  VarLen=11;
  MaxSize=16000;
  MaxStackDeep=1000;
  NFunc=11;
  FuncNames:array[1..NFunc] of funcname=
    ('Abs','Sign','Sin','Cos','Tan','Arcsin','Arccos','Arctan',
     'Ln','Exp','Sqrt');
type
  TVarName=string[VarLen];
  PVarName=^TVarName;
  TNameArray=array[1..MaxSize] of PVarName;
  PNameArray=^TNameArray;

  TStackElement=record
    narg:integer;       {number of arguments for the operation}
    case ElType:char of
      'o': (op:char); {operation: + - * / ^ (integer power)}
      'f': (fn:integer); {function number in the list}
      'r': (r:single); {constant real number}
      'c','v': (n:integer);
        {'c' means named constant; n is its number in the constants array}
        {'v' means variable; n is its number in the variables array}
      't': (); {time variable}
  end;
  PStackElement=^TStackElement;
  TElemArray=array[1..MaxStackDeep] of PStackElement;
  PElemArray=^TElemArray;

  TStack=object
    ar:PElemArray;
    size, nalloc:integer;

    constructor Init;
    destructor Done; virtual;
    function AddElement(var El:TStackElement):boolean;
    function AddPtrElement(var pEl:PStackElement):boolean;
    procedure TrimToSize;
    function ElementAt(n:integer):PStackElement;
    procedure Clear;
    function Push(var el:TStackElement):boolean; {false if result is undefined}
    function Pop:PStackElement;
    procedure Print(var fout:text; var VarNames,ConstNames:PNameArray);
  end;
  PStack=^TStack;

  TStackArray=array[1..MaxSize] of PStack;
  PStackArray=^TStackArray;

  TNumbersArray=array[1..MaxSize] of single;
  PNumbersArray=^TNumbersArray;
  TBoolArray=array[1..MaxSize] of boolean;
  PBoolArray=^TBoolArray;

  TCondition=record
    VarN:integer;
    LowBound, HighBound:single;
    IsLowDefined, IsHighDefined:boolean;
  end;
  PCondition=^TCondition;
  TCondArray=array[1..MaxSize] of PCondition;
  PCondArray=^TCondArray;

  TDynSystem=object
    NVars,NConst:integer;
    VarNames,ConstNames:PNameArray; {memory is allocated by GetMem}
    Formulas:PStackArray; {formulas for the variables; memory is allocated by GetMem}

    {initial state}
    v0,c0:PNumbersArray;
    t0:longint;
    {finish conditions}
    tfin:longint;
    IsTDefined:boolean;
    ncond:integer;
    cond:PCondArray;
    {previous state}
    vprev:PNumbersArray;
    IsPrevUndefined:PBoolArray; {there can be division by zero or}
      {it can be impossible to calculate a function like sqrt or ln}
    {current state}
    tcurr:longint;
    vcurr:PNumbersArray;
    IsCurrUndefined:PBoolArray; {there can be division by zero or}
      {it can be impossible to calculate a function like sqrt or ln}

    constructor Init;
    destructor Done; virtual;
    procedure FreeAll; {disposes all global arrays and structures}
    procedure Print(var fout:text);

    procedure AllocMemForWork;
    procedure NextStep;
    procedure Reset;
    function MustFinish:boolean;
  end;
  PDynSystem=^TDynSystem;

implementation

constructor TStack.Init;
begin
  size:=0; nalloc:=MaxStackDeep;
  GetMem(ar,longint(nalloc)*sizeof(PStackElement));
end;

destructor TStack.Done;
var i:integer;
begin
  if (nalloc>0) and (ar<>nil) then begin
    for i:=1 to size do dispose(ar^[i]);
    FreeMem(ar,longint(nalloc)*sizeof(PStackElement));
  end;
  ar:=nil; size:=0; nalloc:=0;
end;

procedure TStack.Clear;
var i:integer;
begin
  if (nalloc>0) and (ar<>nil) then
    for i:=1 to size do begin
      dispose(ar^[i]); ar^[i]:=nil;
    end;
  size:=0;
end;

function TStack.AddElement(var El:TStackElement):boolean;
begin
  AddElement:=false;
  if size>=nalloc then exit;
  inc(size);
  new(ar^[size]);
  ar^[size]^:=El;
  AddElement:=true;
end;

function TStack.AddPtrElement(var pEl:PStackElement):boolean;
begin
  AddPtrElement:=false;
  if size>=nalloc then exit;
  inc(size);
  ar^[size]:=pEl;
  AddPtrElement:=true;
end;

function ApplyFunction(fnum:integer; arg:single; var result:single):boolean;
{returns false if the fnum-th function is undefined for the argument arg}
{('Abs','Sign','Sin','Cos','Tan','Arcsin','Arccos','Arctan','Ln','Exp','Sqrt');}
var r:real;
begin
  ApplyFunction:=false;
  case fnum of
    1: begin result:=abs(arg); ApplyFunction:=true; end;
    2: begin
      if arg>0 then result:=1 else if arg<0 then result:=-1 else result:=0;
      ApplyFunction:=true;
    end;
    3: begin result:=sin(arg); ApplyFunction:=true; end;
    4: begin result:=cos(arg); ApplyFunction:=true; end;
    5: begin
      r:=cos(arg);
      if r<>0.0 then begin
        result:=sin(arg)/r; ApplyFunction:=true;
      end;
    end;
    6:if (arg>=-1.0) and (arg<=1.0) then begin
        if (arg>-1.0) and (arg<1.0) then
          result:=ArcTan(arg/sqrt(1-sqr(arg)))
        else
          if arg<0 then result:=-PI/2 else result:=PI/2;
        ApplyFunction:=true;
      end;
    7:if (arg>=-1.0) and (arg<=1.0) then begin
        if arg<>0.0 then
          result:=ArcTan(sqrt(1-sqr(arg))/arg)
        else result:=PI/2;
        ApplyFunction:=true;
      end;
    8: begin result:=arctan(arg); ApplyFunction:=true; end;
    9:if arg>0 then begin
        result:=ln(arg); ApplyFunction:=true;
      end;
    10: begin result:=Exp(arg); ApplyFunction:=true; end;
    11: if arg>=0 then begin result:=sqrt(arg); ApplyFunction:=true; end;
  end;
end;

function TStack.Push(var el:TStackElement):boolean; {false if result is undefined}
var
  e1,e2:PStackElement;
  i,n:integer;
  Undefined:boolean;
begin
  Push:=false;
  e1:=nil; e2:=nil;
  if (el.ElType='o') or (el.ElType='f') then begin
    if el.narg>size then exit; {not enough arguments in the stack}
    e2:=Pop; if e2=nil then exit;
    if e2^.ElType<>'r' then begin dispose(e2); exit; end;
    if el.narg>1 then begin
      e1:=Pop;
      if e1=nil then exit;
      if e1^.ElType<>'r' then begin dispose(e1); dispose(e2); exit; end;
    end;
  end;
  case el.ElType of
    'o':
      case el.op of
        '+': begin
          el.ElType:='r';
          if el.narg=1 then el.r:=e2^.r
          else el.r:=e1^.r+e2^.r;
          el.narg:=0;
          AddElement(el); Push:=true;
        end;
        '-': begin
          el.ElType:='r';
          if el.narg=1 then el.r:=-e2^.r
          else el.r:=e1^.r-e2^.r;
          el.narg:=0;
          AddElement(el); Push:=true;
        end;
        '*':
          if el.narg=2 then begin
            el.ElType:='r';
            el.r:=e1^.r*e2^.r;
            el.narg:=0;
            AddElement(el); Push:=true;
          end;
        '/':
          if (el.narg=2) and (e2^.r<>0.0) then begin
            el.ElType:='r';
            el.r:=e1^.r/e2^.r;
            el.narg:=0;
            AddElement(el); Push:=true;
          end;
        '^':
          if el.narg=2 then begin
            n:=round(e2^.r);
            el.ElType:='r';
            el.r:=1;
            Undefined:=false;
            if n>=0 then
              for i:=1 to n do el.r:=el.r*e1^.r
            else begin
              for i:=1 to -n do el.r:=el.r*e1^.r;
              if el.r=0.0 then Undefined:=true else el.r:=1.0/el.r;
            end;
            if not Undefined then begin
              el.narg:=0;
              AddElement(el); Push:=true;
            end;
          end;
      end;
    'f': begin
      el.ElType:='r';
      if ApplyFunction(el.fn,e2^.r,el.r) then begin
        el.narg:=0;
        AddElement(el); Push:=true;
      end;
    end;
    'r': begin {constant real number}
      el.narg:=0;
      AddElement(el); Push:=true;
    end;
  end;
  if e1<>nil then dispose(e1);
  if e2<>nil then dispose(e2);
end;

function TStack.Pop:PStackElement;
begin
  if size<1 then Pop:=nil
  else begin
    Pop:=ar^[size]; ar^[size]:=nil; dec(size);
  end;
end;

procedure TStack.TrimToSize;
var
  i:integer;
  ar1:PElemArray;
begin
  ar1:=nil;
  if (size>0) then GetMem(ar1,longint(size)*sizeof(PStackElement));
  for i:=1 to size do ar1^[i]:=ar^[i];
  FreeMem(ar,longint(nalloc)*sizeof(PStackElement));
  ar:=ar1;
  nalloc:=size;
end;

function TStack.ElementAt(n:integer):PStackElement;
begin
  ElementAt:=nil;
  if (n>0) and (n<=size) then ElementAt:=ar^[n];
end;

procedure TStack.Print(var fout:text; var VarNames,ConstNames:PNameArray);
var i:integer;
begin
  for i:=1 to size do
    with ar^[i]^ do
      case ElType of
        'o': write(fout,' ',op,' ');
        'f': write(fout,' ',FuncNames[fn],' ');
        'r': write(fout,' ',r:6:2,' ');
        'v': write(fout,' ',VarNames^[n]^,' ');
        'c': write(fout,' ',ConstNames^[n]^,' ');
        't': write(fout,' t ');
      end;
  writeln(fout);
end;

constructor TDynSystem.Init;
begin
  NVars:=0; NConst:=0; VarNames:=nil; ConstNames:=nil; Formulas:=nil;
  {initial state}
  v0:=nil; c0:=nil; t0:=0;
  {finish conditions}
  ncond:=0; cond:=nil;
  tfin:=0; IsTDefined:=false;
  {previous state}
  vprev:=nil; IsPrevUndefined:=nil;
  {current state}
  vcurr:=nil; tcurr:=0; IsCurrUndefined:=nil;
end;

destructor TDynSystem.Done;
begin
  FreeAll;
end;

procedure TDynSystem.FreeAll;
var
  i:integer;
begin
  {initial state}
  if v0<>nil then FreeMem(v0,longint(NVars)*sizeof(single)); v0:=nil;
  if c0<>nil then FreeMem(c0,longint(NConst)*sizeof(single)); c0:=nil;
  t0:=0;
  {finish conditions}
  if cond<>nil then begin
    for i:=1 to ncond do dispose(cond^[i]);
    FreeMem(cond,longint(ncond)*sizeof(PCondition));
    cond:=nil; ncond:=0;
  end;
  tfin:=0; IsTDefined:=false;
  {previous state}
  if vprev<>nil then FreeMem(vprev,longint(NVars)*sizeof(single)); vprev:=nil;
  if IsPrevUndefined<>nil then
    FreeMem(IsPrevUndefined,longint(NVars)*sizeof(boolean));
  IsPrevUndefined:=nil;
  {current state}
  if vcurr<>nil then FreeMem(vcurr,longint(NVars)*sizeof(single)); vcurr:=nil;
  if IsCurrUndefined<>nil then
    FreeMem(IsCurrUndefined,longint(NVars)*sizeof(boolean));
  IsCurrUndefined:=nil;
  tcurr:=0;
  if (NVars>0) and (VarNames<>nil) then begin
    for i:=1 to NVars do
      if VarNames^[i]<>nil then dispose(VarNames^[i]);
    FreeMem(VarNames,longint(NVars)*sizeof(PVarName));
  end;
  if (NConst>0) and (ConstNames<>nil) then begin
    for i:=1 to NConst do
      if ConstNames^[i]<>nil then dispose(ConstNames^[i]);
    FreeMem(ConstNames,longint(NConst)*sizeof(PVarName));
  end;
  if (NVars>0) and (Formulas<>nil) then begin
    for i:=1 to NVars do
      if Formulas^[i]<>nil then dispose(Formulas^[i],Done);
    FreeMem(Formulas,longint(NVars)*sizeof(PStack));
  end;
  NVars:=0; NConst:=0; VarNames:=nil; ConstNames:=nil; Formulas:=nil;
end;

procedure TDynSystem.Print(var fout:text);
var i:integer;
begin
  if (VarNames<>nil) and (Formulas<>nil) then
    for i:=1 to NVars do begin
      write(fout,'d',VarNames^[i]^,'/dt=');
      Formulas^[i]^.Print(fout,VarNames,ConstNames);
    end;
end;

procedure TDynSystem.AllocMemForWork;
var i:integer;
begin
  if NVars<=0 then exit;
  GetMem(v0,longint(NVars)*sizeof(single));
  if NConst>0 then GetMem(c0,longint(NConst)*sizeof(single));
  IsTDefined:=false;
  GetMem(vprev,longint(NVars)*sizeof(single));
  GetMem(vcurr,longint(NVars)*sizeof(single));
  GetMem(IsPrevUndefined,longint(NVars)*sizeof(boolean));
  GetMem(IsCurrUndefined,longint(NVars)*sizeof(boolean));
  if c0<>nil then
    for i:=1 to NConst do c0^[i]:=0;
  for i:=1 to NVars do begin
    v0^[i]:=0; vprev^[i]:=0; vcurr^[i]:=0;
    IsPrevUndefined^[i]:=false; IsCurrUndefined^[i]:=false;
  end;
end;

procedure TDynSystem.NextStep;
var
  i,j,k:integer;
  stk:TStack;
  f:PStack;
  OK:boolean;
  stel:TStackElement;
  pst:PStackElement;
begin
  if NVars<=0 then exit;
  inc(tcurr);
  for i:=1 to NVars do begin
    vprev^[i]:=vcurr^[i];
    IsPrevUndefined^[i]:=IsCurrUndefined^[i];
  end;
  stk.Init;
  for i:=1 to NVars do
    if Formulas^[i]<>nil then begin
      {calculate vcurr^[i] by the formula Formulas^[i]^}
      f:=Formulas^[i];
      OK:=true;
      j:=1;
      while OK and (j<=f^.size) do begin
        stel:=f^.ElementAt(j)^;
        case stel.ElType of
          'c': begin k:=stel.n; stel.ElType:='r'; stel.r:=c0^[k]; end;
          'v': begin
            k:=stel.n; stel.ElType:='r';
            if IsPrevUndefined^[k] then OK:=false
            else stel.r:=vprev^[k];
          end;
          't': begin stel.ElType:='r'; stel.r:=tcurr; end;
        end;
        if OK then
          if not stk.Push(stel) then OK:=false
          else inc(j);
      end;
      if OK then begin
        pst:=stk.Pop;
        if pst=nil then IsCurrUndefined^[i]:=true
        else begin
          if pst^.ElType='r' then vcurr^[i]:=pst^.r
          else IsCurrUndefined^[i]:=true;
          dispose(pst);
        end;
      end
      else IsCurrUndefined^[i]:=true;
      stk.Clear;
    end
    else IsCurrUndefined^[i]:=true;
  stk.Done;
end;

procedure TDynSystem.Reset;
var i:integer;
begin
  if NVars<=0 then exit;
  tcurr:=t0;
  for i:=1 to NVars do begin
    vcurr^[i]:=v0^[i];
    IsCurrUndefined^[i]:=false;
  end;
end;

function TDynSystem.MustFinish:boolean;
var i:integer;
begin
  MustFinish:=true;
  if tcurr>=tfin then exit;
  if cond<>nil then
    for i:=1 to ncond do
      with cond^[i]^ do begin
        if (IsLowDefined or IsHighDefined) and (VarN>0) and (VarN<=NVars)
        then
          if not IsCurrUndefined^[VarN] and
             (not IsLowDefined or (vcurr^[VarN]>=LowBound)) and
             (not IsHighDefined or (vcurr^[VarN]<=HighBound))
          then exit;
      end;
  MustFinish:=false;
end;

end.

