unit ExMdi;

{$R EXMDI.RES}

interface

uses WinTypes, WinProcs, OWindows, ODialogs, Strings, TpString,
     CommBtn;

type
  PStatusWindow=^TStatusWindow;
  TStatusWindow=object(TWindow)
    GrayPen,WhitePen:HPen;
    Height,Width:Integer;
    DefaultText:array[0..80] of Char;
    Font:HFont;
    Command:word;
    constructor Init (AParent:PWindowsObject; ACommand:word);
    destructor Done; virtual;
    function GetClassName:PChar; virtual;
    procedure GetWindowClass (var WndClass:TWndClass); virtual;
    procedure Paint (PaintDC:hDC; var PS:TPaintStruct); virtual;
    procedure SetText (txt:PChar);
    procedure WMLButtonDown (var Msg:TMessage); virtual wm_First+wm_LButtonDown;
  end;

  PRibbonWindow=^TRibbonWindow;
  TRibbonWindow=object(TWindow)
    GrayPen,WhitePen:HPen;
    Height:Integer;
    HintText,DefaultText:array[0..80] of Char;
    Font:HFont;
    constructor Init (AParent:PWindowsObject);
    destructor Done; virtual;
    function GetClassName:PChar; virtual;
    procedure GetWindowClass (var WndClass:TWndClass); virtual;
    procedure Paint (PaintDC:hDC; var PS:TPaintStruct); virtual;
    procedure SetHelpText (Txt:PChar);
    procedure WMSetText (var Msg:TMessage); virtual wm_First+wm_SetText;
  end;

  PSpeedButton=^TSpeedButton;
  TSpeedButton=object(TCommBtn)
    BMP:hBitmap;
    Width,Height:word;
    constructor Init(AParent:PWindowsObject; AnID:Integer; x,y:Integer;
                     ABitMap:PChar; AComment:string);
    destructor Done; virtual;
    function GetClassName:PChar; virtual;
    procedure WMLButtonDown (var Msg:TMessage); virtual wm_First+wm_LButtonDown;
    procedure WMLButtonUp (var Msg:TMessage); virtual wm_First+wm_LButtonUp;
  end;

  PSpeedBar=^TSpeedBar;
  TSpeedBar=object(TWindow)
    StatusLine:PWindowsObject;
    Btns:array[1..20] of PSpeedButton;
    NBtns:integer;
    constructor Init (AParent:PWindowsObject; AStatusLine:PWindowsObject);
    function GetClassName:PChar; virtual;
    procedure GetWindowClass (var WndClass:TWndClass); virtual;
    procedure AddATool (AnID:Word; ABitMap:PChar; AComment:string);
    procedure AddSeparator;
    procedure WMDrawItem (var Msg:TMessage); virtual wm_First+wm_DrawItem;
    procedure WMSetFocus (var Msg:TMessage); virtual wm_First+wm_SetFocus;
    procedure WMCommand (var Msg:TMessage); virtual wm_First+wm_Command;
    procedure CloseComment;
  end;

const
  cm_HelpContents = 301;
  cm_HelpHelp     = 302;

type
  PExMDIWindow=^TExMDIWindow;
  TExMDIWindow=object(TMDIWindow)
    StatusWin:array[0..4] of PStatusWindow;
    StatusLine:PRibbonWindow;
    SpeedBar:PSpeedBar;
    Help:Boolean;
    HelpFileName:PChar;
    constructor Init (ATitle:Pchar; AMenu:THandle);
    destructor Done; virtual;
    procedure WMSize (var Msg:TMessage); virtual wm_First+wm_Size;
    procedure WMMenuSelect (var Msg:TMessage); virtual wm_First+wm_MenuSelect;
    procedure CreateSpeedbar; virtual;
    procedure UpdateSpeedbar (var Msg:TMessage);
    procedure GetStatusText (ID:Word; Buffer:PChar; BufferMax:Integer); virtual;
    function GetProgramCopyright:string; virtual;
    {functions used in Help subsystem}
    procedure WMCommand      (var Msg:TMessage); virtual wm_First + wm_Command;
    procedure WMKeyDown      (var Msg:TMessage); virtual wm_First + wm_KeyDown;
    procedure WMEnterIdle    (var Msg:TMessage); virtual wm_First + wm_EnterIdle;
    procedure WMDestroy      (var Msg:TMessage); virtual wm_First + wm_Destroy;
    procedure CMHelpContents (var Msg:TMessage); virtual cm_First + cm_HelpContents;
    procedure CMHelpHelp     (var Msg:TMessage); virtual cm_First + cm_HelpHelp;
    procedure DlgHelp (var Msg:TMessage); virtual wm_First+ 198;
    procedure SCNext (var Msg:TMessage); virtual cm_First + 991;
    procedure SCPrev (var Msg:TMessage); virtual cm_First + 992;
    function GetHelpCtx (WParam:Word) : integer; virtual;
    function GetActiveChild : HWND;
  end;

  PMyMDIChild=^TMyMDIChild;
  TMyMDIChild=object(TWindow)
    ChildId:integer;
    procedure SetupWindow; virtual;
    procedure WMSysCommand (var Msg:TMessage); virtual wm_First + wm_SysCommand;
    procedure Help (var Msg:TMessage); virtual cm_first+1011;
  end;

  PExDlg=^TExDlg;
  TExDlg=object(TDialog)
    HelpCtx:integer;
    procedure Help (var Msg:TMessage); virtual id_first+998;
  end;

implementation

constructor TStatusWindow.Init (AParent:PWindowsObject; ACommand:word);
var DC:hDC;
    OldFont:HFont;
    TM:TTExtMetric;
    LogFont:TLogFont;
    str:string[80];
begin
  { basic initialization for a non-MDI Window in an MDI Frame }
  Inherited Init(AParent,'');      { create the window normally }
  SetFlags(wb_MDIChild,False);     { turn off the MDI flag that TWindow set }
  DefaultProc:=@DefWindowProc;     { and redirect the DefaultProc back }
  Attr.Style:=ws_Border or ws_Child or ws_Visible;

  { establish Font for use in this window }
  Font:=GetStockObject(ANSI_VAR_FONT);
  GetObject(Font,Sizeof(TLogFont),@LogFont);
  LogFont.lfWeight:=700;
  Font:=CreateFontIndirect(LogFont);

  { Determine the height of this window }
  DC:=GetDC(0);
  OldFont:=SelectObject(DC,Font);
  GetTextMetrics(DC,TM);
  Height:=TM.tmHeight+8;
  Width:=Height;
  SelectObject(DC,OldFont);
  ReleaseDC(0,DC);

  { Make the pens and brushes used to draw the status line }
  GrayPen:=CreatePen(PS_SOLID,1,$00808080);
  WhitePen:=CreatePen(PS_Solid,1,$00FFFFFF);

  DefaultText[0]:=#0; Command:=ACommand;
end;

destructor TStatusWindow.Done;
begin
  DeleteObject(GrayPen);
  DeleteObject(WhitePen);
  DeleteObject(Font);
  inherited Done;
end;

function TStatusWindow.GetClassName;
begin
  GetClassName:='StatusWindow';
end;

procedure TStatusWindow.GetWindowClass(var WndClass:TWndClass);
begin
  inherited GetWindowClass(WndClass);
  with WndClass do
    hbrBackGround:=GetStockObject(LtGray_Brush);
end;

procedure TStatusWindow.Paint (PaintDC:hDC; var PS:TPaintStruct);
var OldFont:HFont;
    OldPen :hPen;
    R:TRect;
begin
  GetClientRect(hWindow,R);
  with R do begin
    Inc(Left,4); Inc(top,2); Dec(Right,4); Dec(Bottom,3);
    SetBKMode(PaintDC, Transparent);
    OldPen:=SelectObject(PaintDC, GrayPen);
    MoveTo(PaintDC,left,bottom); LineTo(PaintDC,left,top); LineTo(PaintDC,right,top);
    SelectObject(PaintDC,WhitePen);
    LineTo(PaintDC,right,bottom); LineTo(PaintDC,left,bottom);
    SelectObject(PaintDC,OldPen);
    SetTextColor(PaintDC,0);
    OldFont:=SelectObject(PaintDC,Font);
    IntersectClipRect(PaintDC, left, top, right-2, bottom);
  end;
  TextOut(PaintDC,8,3,DefaultText,StrLen(DefaultText));
  SelectObject(PaintDC,OldFont);
end;

procedure TStatusWindow.SetText (txt:PChar);
begin
  if Txt=nil then
    DefaultText[0]:=#0
  else
    StrLCopy(DefaultText,Txt,80);
  if hWindow<>0 then InvalidateRect(hWindow,nil,True);
end;

procedure TStatusWindow.WMLButtonDown (var Msg:TMessage);
begin
  if Command<>0 then
    SendMessage(Parent^.HWindow,wm_Command,Command,0);
end;

{---------------------------------------}
constructor TRibbonWindow.Init (AParent:PWindowsObject);
var DC:hDC;
    OldFont:HFont;
    TM:TTExtMetric;
    LogFont:TLogFont;
    str:string[80];
begin
  { basic initialization for a non-MDI Window in an MDI Frame }
  Inherited Init(AParent,'');      { create the window normally }
  SetFlags(wb_MDIChild,False);     { turn off the MDI flag that TWindow set }
  DefaultProc:=@DefWindowProc;     { and redirect the DefaultProc back }
  Attr.Style:=ws_Border or ws_Child or ws_Visible;

  { establish Font for use in this window }
  Font:=GetStockObject(ANSI_VAR_FONT);
  GetObject(Font,Sizeof(TLogFont),@LogFont);
  LogFont.lfWeight:=700;
  Font:=CreateFontIndirect(LogFont);

  { Determine the height of this window }
  DC:=GetDC(0);
  OldFont:=SelectObject(DC,Font);
  GetTextMetrics(DC,TM);
  Height:=TM.tmHeight+8;
  SelectObject(DC,OldFont);
  ReleaseDC(0,DC);

  { Make the pens and brushes used to draw the status line }
  GrayPen:=CreatePen(PS_SOLID,1,$00808080);
  WhitePen:=CreatePen(PS_Solid,1,$00FFFFFF);

  { Set up the text displayed in the hint line when there's nothing
    of greater importance to display }
  str:=PExMDIWindow(Parent)^.GetProgramCopyright;
  StrPCopy(DefaultText,str);
  HintText[0]:=#0;
end;

destructor TRibbonWindow.Done;
begin
  DeleteObject(GrayPen);
  DeleteObject(WhitePen);
  DeleteObject(Font);
  inherited Done;
end;

function TRibbonWindow.GetClassName;
begin
  GetClassName:='RibbonWindow';
end;

procedure TRibbonWindow.GetWindowClass(var WndClass:TWndClass);
begin
  inherited GetWindowClass(WndClass);
  with WndClass do
    hbrBackGround:=GetStockObject(LtGray_Brush);
end;

procedure TRibbonWindow.Paint (PaintDC:hDC; var PS:TPaintStruct);
var OldFont:HFont;
    OldPen :hPen;
    R:TRect;
begin
  GetClientRect(hWindow,R);
  with R do begin
    Inc(Left,4); Inc(top,2); Dec(Right,4); Dec(Bottom,3);
    SetBKMode(PaintDC, Transparent);
    OldPen:=SelectObject(PaintDC, GrayPen);
    MoveTo(PaintDC,left,bottom); LineTo(PaintDC,left,top); LineTo(PaintDC,right,top);
    SelectObject(PaintDC,WhitePen);
    LineTo(PaintDC,right,bottom); LineTo(PaintDC,left,bottom);
    SelectObject(PaintDC,OldPen);
    SetTextColor(PaintDC,0);
    OldFont:=SelectObject(PaintDC,Font);
    IntersectClipRect(PaintDC, left, top, right-2, bottom);
  end;
  if HintText[0]<>#0 then
    TextOut(PaintDC,8,3,HintText,StrLen(HintText))
  else
    TextOut(PaintDC,8,3,DefaultText,StrLen(DefaultText));
  SelectObject(PaintDC,OldFont);
end;

procedure TRibbonWindow.SetHelpText (Txt:PChar);
begin
  if Txt=nil then
    HintText[0]:=#0
  else
    StrLCopy(HintText,Txt,80);
  if hWindow<>0 then InvalidateRect(hWindow,nil,True);
end;

procedure TRibbonWindow.WMSetText (var Msg:TMessage);
begin
  SetHelpText(PChar(Msg.lPAram));
end;

{---------------------------------------}
constructor TSpeedButton.Init (AParent:PWindowsObject; AnID:Integer;
                               x,y:Integer; ABitMap:PChar; AComment:string);
var B:TBitMap;
begin
  BMP:=LoadBitMap(hInstance,ABitMap);
  if BMP=0 then Fail;
  GetObject(BMP,Sizeof(B),@B);
  Width:=B.bmWidth; Height:=B.bmHeight;
  inherited Init(AParent,AnID,x,y,Width,Height,AComment);
  Attr.Style:=Attr.Style or bs_OwnerDraw;
end;

destructor TSpeedButton.Done;
begin
  inherited Done;
  if BMP<>0 then DeleteObject(BMP);
end;

function TSpeedButton.GetClassName:PChar;
begin
  GetClassName:='Button';
end;

procedure TSpeedButton.WMLButtonDown (var Msg:TMessage);
var HelpText:array[0..80] of Char;
begin
  if PSpeedBar(Parent)^.StatusLine<>nil then begin
    HelpText[0]:=#0;
    {LoadString(hInstance,Attr.ID,HelpText,80);}
    PExMDIWindow(Parent^.Parent)^.GetStatusText(Attr.ID,HelpText,80);
    SendMessage(PSpeedBar(Parent)^.StatusLine^.hWindow,
                wm_SetText,0,LongInt(@HelpText));
  end;
  inherited WMLButtonDown(Msg);
  DefWndProc(Msg);
end;

procedure TSpeedButton.WMLButtonUp (var Msg:TMessage);
begin
  if PSpeedBar(Parent)^.StatusLine<>nil then
    SendMessage(PSpeedBar(Parent)^.StatusLine^.hWindow,wm_SetText,0,0);
  DefWndProc(Msg);
end;

{ Speedbar Methods }

constructor TSpeedBar.Init (AParent:PWindowsObject; AStatusLine:PWindowsObject);
var i:Integer;
begin
  inherited Init(AParent,'');
  StatusLine:=AStatusLine;
  Flags:=Flags and (not wb_MDIChild);
  DefaultProc:=@DefWindowProc;
  Attr.X:=0; Attr.Y:=0; Attr.W:=2; Attr.H:=0;
  Attr.Style:=ws_Visible or ws_Child or ws_Border;
  for i:=1 to 20 do Btns[i]:=nil;
  NBtns:=0;
end;

function TSpeedBar.GetClassName:PChar;
begin
  GetClassName:='SpeedBar';
end;

procedure TSpeedBar.GetWindowClass(var WndClass:TWndClass);
begin
  inherited GetWindowClass(WndClass);
  with WndClass do
    hbrBackGround:=GetStockObject(LtGray_Brush);
end;

procedure TSpeedBar.AddATool (AnID:Word; ABitMap:PChar; AComment:string);
var Tool:PSpeedButton;
    NewWidth,i,x,y,Col:Integer;
begin
  x:=Attr.W; y:=2;
  Tool:=New(PSpeedButton,Init(@Self,AnID,x,y,ABitMap,AComment));
  if Tool=nil then exit;
  inc(NBtns); Btns[NBtns]:=Tool;
  Inc(Attr.W,(Tool^.Width+1));
  if Attr.H<(Tool^.Height+6) then Attr.H:=Tool^.Height+6;
end;

procedure TSpeedBar.AddSeparator;
begin
  inc(Attr.W,5);
end;

procedure TSpeedBar.CloseComment;
var i:integer;
begin
  for i:=1 to NBtns do
    if Btns[i]<>nil then
      Btns[i]^.DeleteComment;
end;

procedure TSpeedBar.WMDrawItem (var Msg:TMessage);
var Tool:PSpeedButton;
    MemDC:hDC;
    OldBMP:hBitmap;
    x,y:Integer;
begin
  with PDrawItemStruct(Msg.lParam)^ do begin
    Tool:=PSpeedButton(ChildWithID(CtlID));
    MemDC:=CreateCompatibleDC(hDC);
    OldBMP:=SelectObject(MemDC,Tool^.BMP);
    if (ItemState and ODS_Selected) <> 0 then begin
      x:=1; y:=1;
    end
    else begin
      x:=0; y:=0;
    end;
    BitBlt(hDC,x,y,Tool^.Width-x*2,Tool^.Height-y*2,MemDC,0,0,SrcCopy);
    SelectObject(MemDC,OldBMP);
    DeleteDC(MemDC);
  end;
end;

procedure TSpeedBar.WMSetFocus (var Msg:TMessage);
begin
  SetFocus(Msg.wParam);
end;

function DescendantOf (AncestorType, ThisObj:Pointer):Boolean;
type
  TVMT=record
    dwSize:LongInt;
    DMTOfs:Word;
  end;
var DMTOfs, TargetDMT:Word;
begin
  DescendantOf:=True;
  DMTOfs:=TVMT(ThisObj^).DMTOfs;
  TargetDMT:=TVMT(AncestorType^).DMTOfs;
  while DMTOfs <> 0 do begin
    if DMTOfs=TargetDMT then Exit;
    DMTOfs:=Word(Ptr(DSeg, DMTOfs)^);
  end;
  DescendantOf:=False;
end;

procedure TSpeedBar.WMCommand (var Msg:TMessage);
var MDIChild,MDIClient:hWnd;
begin
  if DescendantOf(TypeOf(TMDIWindow),TypeOf(Application^.MainWindow^)) then begin
    MDIClient:=PMDIWindow(Application^.MainWindow)^.ClientWnd^.hWindow;
    MDIChild:=SendMessage(MDIClient,WM_MDIGetActive,0,0);
    if MDIChild=0 then MDIChild:=MDIClient;
  end
  else
    MDIChild:=Application^.MainWindow^.hWindow;
  SendMessage(MDIChild,WM_COMMAND,Msg.wParam,0);
end;

{---------------------------------------------}
const ExeNameMaxSize = 128;

constructor TExMDIWindow.Init(ATitle:Pchar; AMenu:THandle);
var FileNameLen:integer;
    FileName:array[0..ExeNameMaxSize + 1] of Char;
    I:Integer;
begin
  inherited Init(ATitle,AMenu);
  Attr.X:=20; Attr.Y:=20;
  Attr.W:=GetSystemMetrics(SM_CXFULLSCREEN)-40;
  Attr.H:=GetSystemMetrics(SM_CYFULLSCREEN)-40;

  for i:=0 to 4 do StatusWin[i]:=nil;

  StatusLine:=New(PRibbonWindow,Init(@Self));
  SpeedBar:=nil; CreateSpeedBar;

  (*
  { Construct HelpFileName from Module Name }
  FileNameLen:=GetModuleFileName(HInstance,FileName,ExeNameMaxSize);
  I:=FileNameLen - 1;
  while (I<>0) and ((FileName[I]<>'\') and (FileName[I]<>':')) do Dec(I);
  Inc(I);
  if I + 13 <= ExeNameMaxSize then
    StrCopy(@FileName[I],'helpex.hlp')
  else
    StrCopy(@FileName[I],'?'); *)
  HelpFileName:=StrNew('trax.hlp'); Help:=False;
end;

destructor TExMDIWindow.Done;
begin
  StrDispose(HelpFileName);
  inherited Done;
end;

procedure TExMDIWindow.CreateSpeedBar;
begin
  if SpeedBar<>nil then SpeedBar^.Free;
  SpeedBar:=New(PSpeedBar,Init(@Self,StatusLine));

  with SpeedBar^ do begin
    AddATool(cm_HelpContents,Pchar('BtnHelp'),'');
    AddSeparator;
  end;
end;

procedure TExMDIWindow.UpdateSpeedBar (var Msg:TMessage);
var Rect:TRect;
    SizeMsg:TMessage;
begin
  if SpeedBar<>nil then
    SpeedBar^.Free;
  CreateSpeedBar;
  Application^.MakeWindow(SpeedBar);
  GetClientRect(hWindow,Rect);
  SizeMsg.lParamLo:=Rect.Right; SizeMsg.lParamHi:=Rect.Bottom;
  WMSize(SizeMsg);
end;

procedure TExMDIWindow.WMSize (var Msg:TMessage);
var ClientX,ClientY,StatusX,StatusY,SpeedX,SpeedY,ClientW,ClientH,
    StatusW,StatusH,SpeedW,SpeedH,SumW,i:integer;
begin
  ClientX:=0; ClientY:=0; ClientW:=Msg.lParamLo; ClientH:=Msg.lParamHi;
  SumW:=0;
  for i:=0 to 4 do
    if StatusWin[i]<>nil then
      with StatusWin[i]^ do begin
        StatusX:=ClientX-1+SumW; StatusY:=ClientH-Height;
        if StatusWin[i]^.hWindow <> 0 then begin
          MoveWindow(StatusWin[i]^.hWindow,StatusX,StatusY,
                     Width+1,StatusLine^.Height+1,True);
          inc(SumW,Width+1);
        end;
      end;
  if StatusLine<>nil then
    with StatusLine^ do begin
      StatusX:=ClientX-1+SumW; StatusY:=ClientH-Height;
      StatusW:=ClientW+2-SumW; StatusH:=Height+1;
      Dec(ClientH,Height);
      if StatusLine^.hWindow <> 0 then
        MoveWindow(StatusLine^.hWindow,StatusX,StatusY,StatusW,StatusH,True);
    end;
  if SpeedBar<>nil then begin
    SpeedX:=ClientX; Dec(ClientY);
    SpeedY:=ClientY; SpeedW:=ClientW+1; SpeedH:=SpeedBar^.Attr.H;
    Inc(ClientY,SpeedH); Dec(ClientH,Pred(SpeedH));
    if SpeedBar^.hWindow<>0 then
      MoveWindow(SpeedBar^.hWindow,SpeedX,SpeedY,SpeedW,SpeedH,True);
  end;
  if (ClientWnd<>nil) and (ClientWnd^.HWindow<>0) then
    MoveWindow(ClientWnd^.HWindow, ClientX,ClientY,ClientW,ClientH,True);
end;

procedure TExMDIWindow.WMMenuSelect (var Msg:TMessage);
var HelpText:array[0..80] of char;
    I:integer;
    StrIdx:word;
    NItems:integer;
    Popuphandles:array[-2..20] of word;
begin
  if StatusLine=nil then exit;
  NItems:=GetMenuItemCount(Attr.Menu);
  PopupHandles[-2]:=ClientWnd^.ClientAttr.hWindowMenu;
  PopupHandles[-1]:=GetSystemMenu(HWindow,false);
  for i:=0 to NItems do
    PopupHandles[i]:=GetSubMenu(Attr.Menu,i);
  HelpText[0]:=#0;
  if Msg.LParamLo<>$FFFF then begin
    StrIdx:=Msg.wParam;
    if (Msg.lParamLo and MF_POPUP)<>0 then begin
      I:=-2; StrIdx:=0;
      repeat
        if PopupHandles[I]=Msg.wParam then StrIdx:=1000+i;
        inc(I);
      until (I>NItems) or (StrIdx<>0);
    end;
    if (StrIdx>=$0F00) and (StrIdx<=$0F0A) then
      StrCopy(HelpText,'  ') {???}
    else  if (StrIdx<>0) {and (StrIdx<$F000)} then
      GetStatusText(StrIdx,HelpText,80);
  end;
  StatusLine^.SetHelpText(HelpText);
  DefWndProc(Msg);
end;

procedure TExMDIWindow.GetStatusText (ID:Word; Buffer:PChar; BufferMax:Integer);
var str:string;
begin
  case ID of
    991: str:=' ';
    992: str:=' ';
    999: str:=' ';
    61728: str:='  ';
    61456: str:=' ';
    61440: str:='  ';
    61472: str:='  ';
    61488: str:='   ';
    61536: str:=' ';
    61504: str:='   ';
    61744: str:='   ';
    24340: str:='   ';
    cm_ArrangeIcons:    str:=' '; {24335}
    cm_TileChildren:    str:='   ';    {24336}
    cm_CascadeChildren: str:='   ';        {24337}
    cm_CloseChildren:   str:='  ';       {24338}
    else str:='';
  end;
  str:=copy(str,1,BufferMax-1);
  StrPCopy(Buffer,str);
end;

function TExMDIWindow.GetProgramCopyright:string;
begin
  GetProgramCopyright:='The program';
end;

procedure TExMDIWindow.WMCommand (var Msg:TMessage);
var HelpContextId:LongInt;
    pc:array[0..80] of char;
begin
  if Help then begin
    { Determine the Appropriate Help Context }
    HelpContextId:=GetHelpCtx(Msg.wParam);

    { Warn of No Context Help Available or Let the Help System take over. }
    if HelpContextId = 0 then begin
      MessageBox(HWindow,
                 StrPCopy(pc,'Help not available for this ('+long2str(Msg.WParam)+') Menu Item'),
                 nil,Mb_Ok or Mb_IconInformation);
      Help:=false;
      DefWndProc(Msg);
    end
    else begin
      Help:=False;
      WinHelp(HWindow,HelpFileName,Help_Context,HelpContextId);
    end;
  end
  else begin
    if Speedbar<>nil then SpeedBar^.CloseComment;
    inherited WMCommand(Msg);
  end;
end;

{ Responds to normal keypresses }
procedure TExMDIWindow.WMKeyDown (var Msg:TMessage);
begin
  if Msg.wParam=vk_F1 then
    if GetKeyState(vk_Shift)<0 then begin
      Help:=True;
      DefWndProc(Msg);
    end
    else WinHelp(HWindow,HelpFileName,Help_Index,0)
  else;
end;

{ Select Help during Idle }
procedure TExMDIWindow.WMEnterIdle (var Msg:TMessage);
begin
  if (Msg.wParam=msgf_Menu) and ((GetKeyState(vk_F1) and $8000)<>0) then begin
    Help:=True;
    PostMessage(HWindow,wm_KeyDown,vk_Return,0);
  end;
end;

{ Shut Down the Help System and post Quit }
procedure TExMDIWindow.WMDestroy (var Msg:TMessage);
begin
  WinHelp(HWindow,HelpFileName,help_Quit,0);
  PostQuitMessage(0);
end;

procedure TExMDIWindow.CMHelpContents (var Msg:TMessage);
begin
  WinHelp(HWindow,HelpFileName,Help_Index,0);
end;

procedure TExMDIWindow.CMHelpHelp (var Msg:TMessage);
begin
  WinHelp(HWindow,'WINHELP.HLP',Help_Index,0);
end;

function TExMDIWindow.GetHelpCtx (WParam:Word) : integer;
begin
  GetHelpCtx:=0;
end;

function TExMDIWindow.GetActiveChild : HWND;
begin
  GetActiveChild:=SendMessage(ClientWnd^.HWindow,WM_MDIGetActive,0,0);
end;

procedure TExMDIWindow.SCNext (var Msg:TMessage);
begin
  SendMessage(ClientWnd^.hWindow,WM_MDInext,0,0);
end;

procedure TExMDIWindow.SCPrev (var Msg:TMessage);
begin
  SendMessage(ClientWnd^.hWindow,WM_MDInext,0,1);
end;

procedure TExMDIWindow.DlgHelp (var Msg:TMessage);
begin
  Help:=False;
  WinHelp(HWindow,HelpFileName,Help_Context,Msg.WParam);
end;

{-------------------------------------}

procedure TMyMDIChild.SetupWindow;
var hm:HMenu;
begin
  inherited SetupWindow;
  hm:=GetSystemMenu(HWindow,false);
  DeleteMenu(hm,SC_TASKLIST,MF_ByCommand);
  DeleteMenu(hm,61536,MF_ByCommand);
  Deletemenu(hm,GetMenuItemCount(hm)-1,MF_ByPosition);
  AppendMenu(hm,MF_STRING,61536,' ');
  AppendMenu(hm,MF_SEPARATOR,0,'');
  AppendMenu(hm,MF_STRING,991,' ');
  AppendMenu(hm,MF_STRING,992,' ');
end;

procedure TMyMDIChild.WmSysCommand (var Msg: TMessage);
var r:TRect;
begin
  case Msg.WParam of
    991,992:
      SendMessage(Parent^.HWindow,WM_Command,Msg.WParam,0);
    61488,61490:
      begin
        GetClientRect(PExMDIWindow(Parent)^.ClientWnd^.HWindow,r);
        OpenIcon(HWindow);
        with r do MoveWindow(HWindow,0,0,right,bottom,true);
        Msg.WParam:=0;
      end;
    else
      inherited WmSysCommand(Msg);
  end;
end;

procedure TMyMdiChild.Help (var Msg:TMessage);
begin
  SendMessage(Parent^.HWindow,198,0,0);
end;

{-------------------------------------}

procedure TExDlg.Help (var Msg:TMessage);
begin
  if Parent^.Parent=nil then
    SendMessage(Parent^.HWindow,198,HelpCtx,0)
  else
    SendMessage(Parent^.Parent^.HWindow,198,HelpCtx,0);
end;

begin
end.
