{$A+,B-,D+,E+,F+,G-,I+,L+,N+,O+,P-,Q+,R+,S+,T-,V-,X+,Y+}
{$M 1024,0,0}

unit ParsUnit;

interface

uses TPString,DynSys;

type
  TGetInitValues=object
    pds:PDynSystem;
    IsVDefined,IsCDefined:PBoolArray;

    constructor Init;
    destructor Done;
    procedure SetDynSystem(pDynSys:PDynSystem);
    function AreAllDefined:boolean;
    function ReadValueFromString(str:string):boolean;
      {the string has the format: <identifier>=<number>}
    function ReadValuesFromFile(var fin:text):boolean;
    function ReadConditionFromString(str:string):PCondition;
      {permissible formats: x [=,>,<,>=,<=] n; n [=,>,<,>=,<=] x;
       n1 [<,<=] x [<,<=] n2, where x-identifier, n,n1,n2-numbers}
      {< is equal to <=; > is equal to >=}
    procedure ReadFinishConditionsFromFile(var fin:text);
  end;

function parse(var fin,fout:text):PDynSystem; {if errors returns nil}

implementation

function CheckId(var Id:string):boolean;
const bias=ord('a')-ord('A');
var i,len:integer;
begin
  CheckId:=false;
  Id:=trim(Id);
  if Id='' then exit;
  len:=length(Id);
  if len>VarLen then begin Id:=copy(Id,1,VarLen); len:=VarLen; end;
  Id[1]:=UpCase(Id[1]);
  if not ((Id[1]>='A') and (Id[1]<='Z')) then exit;
  for i:=2 to len do begin
    if (Id[i]>='0') and (Id[i]<='9') then
    else
      if (Id[i]>='a') and (Id[i]<='z') then
      else
        if (Id[i]>='A') and (Id[i]<='Z') then
          Id[i]:=chr(ord(Id[i])+bias)
        else Exit;
  end;
  CheckId:=true;
end;

function parse(var fin,fout:text):PDynSystem;
const MaxVar=16000;
type
  TNames=array[1..MaxVar] of PVarName;
  TBool=array[1..MaxVar] of boolean;
var
  names:^TNames;
  IsVariable:^TBool;
  nnames:integer;
  nstr:longint;
  stacks:^TStackArray;

  function GetToken(var str:string; var start:integer; var toktype:char):string;
  const
    delim='+-*/^(){}[]; ';
  var
    st,st1:string;
    i,j,k,nn:integer;
    r:real;
  begin
    GetToken:='';
    st:=trim(copy(str,start,256));
    while (st='') and not eof(fin) do begin
      readln(fin,str); inc(nstr); str:=trim(str);
      writeln(fout,nstr,': ',str);
      st:=str; start:=1;
    end;
    if st='' then begin
      writeln(fout,':  䠩 祭 ࠭  ');
      Exit;
    end;
    start:=start+pos(st,copy(str,start,256))-1;
    case st[1] of
      '+','-','*','/','^': begin toktype:='o'; GetToken:=copy(st,1,1); end;
      '(','{','[': begin toktype:='('; GetToken:=copy(st,1,1); end;
      ')','}',']': begin toktype:=')'; GetToken:=copy(st,1,1); end;
      ';':         begin toktype:=';'; GetToken:=copy(st,1,1); end
      else begin
        {looking for the end of the token}
        toktype:='n'; {name}
        {may be, this is a real number in the format nEk}
        nn:=1;
        k:=pos('e',st); j:=pos('E',st);
        if (j>0) and ((k<=0) or (k>j)) then k:=j;
        if (k>0) and Str2Real(copy(st,1,k-1),r) then begin
          inc(k);
          if (st[k]='+') or (st[k]='-') then inc(k);
          if (st[k]>='0') and (st[k]<='9') then nn:=k;
        end;
        j:=0;
        if nn>1 then st1:=copy(st,nn,255) else st1:=st;
        for i:=1 to length(delim) do begin
          k:=pos(copy(delim,i,1),st1);
          if (k>0) then
            if j=0 then j:=k
            else
              if j>k then j:=k;
        end;
        if j>0 then GetToken:=copy(st,1,nn+j-2)
        else GetToken:=st;
      end;
    end;
  end;

  function GetExpression(nvar:integer; var str:string; finchar:char;
                         var currpos:integer):PStack;
  {gets the formula for the nvar-th variable from str and following lines}
  const
    MaxNTerms=100;
  var
    i,k:integer;
    str1,tmpid:string;
    toktype,tt,WhatWasLast:char;
    term:PStack;
    terms:array[1..MaxNTerms] of PStack;
    nterms:integer;
    operations:array[1..MaxNTerms] of char;
    noper:integer;
    stelem:TStackElement;
    pstelem:PStackElement;
    r:real;
    OK:boolean;
  begin
    GetExpression:=nil;
    if currpos=0 then
      currpos:=pos('=',str)+1;
    nterms:=0; noper:=0; WhatWasLast:='0';
    while (str[currpos]<>finchar) and (str[currpos]<>';') do begin
      str1:=GetToken(str,currpos,toktype);
      if str1='' then begin
        writeln(fout,pad(' ',currpos+3),'^^^  ࠦ');
        for i:=1 to nterms do dispose(terms[i],Done);
        exit;
      end;
      if (toktype=finchar) or (toktype=';') then
        str[currpos]:=toktype
      else begin
        case toktype of
          'o': begin
            if WhatWasLast<>'t' then {operation symbol goes not between terms}
              if ((str1<>'+') and (str1<>'-')) or (noper>nterms)
              then begin {this is not an unary operation}
                writeln(fout,pad(' ',currpos+3),'^^^  ࠦ');
                for i:=1 to nterms do dispose(terms[i],Done);
                exit;
              end;
            inc(noper); operations[noper]:=str1[1];
            inc(currpos,length(str1));
          end;
          ')': begin
            writeln(fout,pad(' ',currpos+3),'^^^:  ᪮');
            for i:=1 to nterms do dispose(terms[i],Done);
            exit;
          end;
          '(': begin
            if (WhatWasLast<>'0') and (WhatWasLast<>'o') then begin
              writeln(fout,pad(' ',currpos+3),'^^^:   樨');
              for i:=1 to nterms do dispose(terms[i],Done);
              exit;
            end;
            inc(currpos,length(str1));
            term:=GetExpression(nvar,str,')',currpos);
            if term=nil then begin
              for i:=1 to nterms do dispose(terms[i],Done);
              exit;
            end
            else begin
              inc(nterms);
              terms[nterms]:=term;
            end;
          end
          else begin
            if (WhatWasLast<>'0') and (WhatWasLast<>'o') then begin
              writeln(fout,pad(' ',currpos+3),'^^^:   樨');
              for i:=1 to nterms do dispose(terms[i],Done);
              exit;
            end;
            if Str2Real(str1,r) then begin {this is a number}
              stelem.narg:=0;
              stelem.ElType:='r';
              stelem.r:=r;
              new(term,Init);
              term^.AddElement(stelem);
              inc(nterms);
              terms[nterms]:=term;
              inc(currpos,length(str1));
            end
            else begin
              tmpid:=str1;
              if not CheckId(tmpid) then begin
                writeln(fout,pad(' ',currpos+3),'^^^: ४ 䨪');
                writeln(fout,pad(' ',currpos+3),' 䨪 稭  .㪢  ᮤন  ',
                        VarLen,' ᨬ (.㪢  )');
                for i:=1 to nterms do dispose(terms[i],Done);
                exit;
              end;
              inc(currpos,length(str1));
              {possibly, this is a function name}
              i:=1;
              while (i<=NFunc) and (tmpid<>FuncNames[i]) do inc(i);
              if i>NFunc then begin  {variable or named constant}
                k:=0;
                stelem.narg:=0;
                if (tmpid='N') or (tmpid='T') then {time variable}
                  stelem.ElType:='t'
                else begin
                  k:=1;
                  while (k<=nnames) and (names^[k]^<>tmpid) do inc(k);
                  if k>nnames then begin
                    inc(nnames);
                    new(names^[nnames]); names^[nnames]^:=tmpid;
                    IsVariable^[nnames]:=false;
                    stacks^[nnames]:=nil;
                    k:=nnames;
                  end;
                  stelem.ElType:='v'; stelem.n:=k;
                end;
                new(term,Init);
                term^.AddElement(stelem);
                inc(nterms);
                terms[nterms]:=term;
              end
              else begin {function}
                str1:=GetToken(str,currpos,tt);
                if (str1='') or (tt<>'(') then begin
                  writeln(fout,pad(' ',currpos+3),'^^^:   ᪮');
                  for i:=1 to nterms do dispose(terms[i],Done);
                  exit;
                end;
                inc(currpos,length(str1));
                term:=GetExpression(nvar,str,')',currpos);
                if term=nil then begin
                  for i:=1 to nterms do dispose(terms[i],Done);
                  exit;
                end
                else begin
                  stelem.narg:=1; {are functions with more than 1 argument possible?}
                  stelem.ElType:='f'; stelem.fn:=i;
                  term^.AddElement(stelem);
                  inc(nterms);
                  terms[nterms]:=term;
                end;
              end;
            end;
          end;
        end;
        if (toktype<>'o') and (WhatWasLast='o') and (noper>=nterms) then begin
          if operations[noper]='-' then
            if (terms[nterms]^.size=1) and
               (terms[nterms]^.ElementAt(1)^.ElType='r') then begin
              pstelem:=terms[nterms]^.ElementAt(1);
              pstelem^.r:=-pstelem^.r;
            end
            else begin
              stelem.narg:=1;
              stelem.ElType:='o';
              stelem.op:='-';
              terms[nterms]^.AddElement(stelem);
            end;
          dec(noper);
        end;
        if toktype='o' then WhatWasLast:='o' else WhatWasLast:='t';
      end;
    end;
    OK:=false;
    if str[currpos]<>finchar then
      writeln(fout,pad(' ',currpos+3),'^^^: ⠥ 뢠 ᪮')
    else
    if nterms=0 then
      writeln(fout,pad(' ',currpos+3),'^^^: ⮥ ()ࠦ')
    else
    if nterms<=noper then
      writeln(fout,pad(' ',currpos+3),'^^^:  ')
    else OK:=true;
    if OK then begin
      {processing operation ^}
      i:=1;
      while i<=noper do
        if operations[i]='^' then begin
          {<i-th term>^<i+1-th term>}
          for k:=1 to terms[i+1]^.size do
            terms[i]^.AddElement(terms[i+1]^.ElementAt(k)^);
          stelem.narg:=2;
          stelem.ElType:='o';
          stelem.op:=operations[i];
          terms[i]^.AddElement(stelem);
          dispose(terms[i+1],Done);
          for k:=i+2 to nterms do terms[k-1]:=terms[k];
          dec(nterms);
          for k:=i+1 to noper do operations[k-1]:=operations[k];
          dec(noper);
        end
        else inc(i);
      {processing operations * and /}
      i:=1;
      while i<=noper do
        if (operations[i]='*') or (operations[i]='/') then begin
          {<i-th term>*<i+1-th term>}
          for k:=1 to terms[i+1]^.size do
            terms[i]^.AddElement(terms[i+1]^.ElementAt(k)^);
          stelem.narg:=2;
          stelem.ElType:='o';
          stelem.op:=operations[i];
          terms[i]^.AddElement(stelem);
          dispose(terms[i+1],Done);
          for k:=i+2 to nterms do terms[k-1]:=terms[k];
          dec(nterms);
          for k:=i+1 to noper do operations[k-1]:=operations[k];
          dec(noper);
        end
        else inc(i);
      {processing operations + and -}
      while (noper>0) and (nterms>1) do begin
        {<1st term>+<2nd term>}
        for k:=1 to terms[2]^.size do
          terms[1]^.AddElement(terms[2]^.ElementAt(k)^);
        stelem.narg:=2;
        stelem.ElType:='o';
        stelem.op:=operations[1];
        terms[1]^.AddElement(stelem);
        dispose(terms[2],Done);
        for k:=3 to nterms do terms[k-1]:=terms[k];
        dec(nterms);
        for k:=2 to noper do operations[k-1]:=operations[k];
        dec(noper);
      end;
      GetExpression:=terms[1];
    end
    else
      for i:=1 to nterms do dispose(terms[i],Done);
    inc(currpos);
  end;

type
  NumArray=array[1..MaxSize] of integer;
var
  str,str1,tmpid:string;
  nform,nerrors,nerrprev:integer;
  i,j,k,lpos,nvar:integer;
  stk:PStack;
  pstel:PStackElement;
  numbers:^NumArray;
  AllExpressionsOK:boolean;
  pds:PDynSystem;
  NVars,NConst:integer;
begin
  pds:=nil;
  nstr:=0; nform:=0; nnames:=0; nerrors:=0;
  new(names); new(IsVariable); new(stacks);
  for i:=1 to MaxSize do stacks^[i]:=nil;
  AllExpressionsOK:=true;
  while not eof(fin) do begin
    repeat
      readln(fin,str); inc(nstr); str:=trim(str);
      writeln(fout,nstr,': ',str);
    until eof(fin) or (str<>'');
    if str<>'' then begin
      nerrprev:=nerrors;
      {find and analyse the left side of the formula}
      nvar:=0;
      i:=pos('/',str); j:=0;
      if i>0 then begin
        str1:=trim(copy(str,i+1,256));
        j:=pos('=',str1);
      end;
      if (i<=0) or (j<=0) or
         ((str[1]<>'d') and (str[1]<>'D')) or
         ((str1[1]<>'d') and (str1[1]<>'D'))
      then begin
        writeln(fout,'   ^^^: 㫠  稭  dX/dt=,  X - 䨪');
        inc(nerrors);
      end
      else begin
        tmpid:=copy(str1,2,j-2);
        if (tmpid<>'t') and (tmpid<>'T') and (tmpid<>'n') and (tmpid<>'N') then begin
          lpos:=pos('=',str);
          writeln(fout,pad(' ',lpos-2),'   ^^^: ६ ६    t  n');
          inc(nerrors);
        end
        else begin
          tmpid:=trim(copy(str,2,i-2));
          if not CheckId(tmpid) then begin
            writeln(fout,'    ^^^: ४⭮  ६');
            writeln(fout,'       ६ 稭  .㪢  ᮤন  ',
                    VarLen,' ᨬ (.㪢  )');
            inc(nerrors);
          end
          else begin
            i:=1;
            while (i<=nnames) and (names^[i]^<>tmpid) do inc(i);
            if i>nnames then begin
              inc(nnames);
              new(names^[nnames]); names^[nnames]^:=tmpid;
              IsVariable^[nnames]:=true;
              stacks^[nnames]:=nil;
              nvar:=nnames;
            end
            else
              if IsVariable^[i] then begin
                writeln(fout,'    ^^^: ୠ 㫠  ६ ',tmpid);
                inc(nerrors);
              end
              else begin
                IsVariable^[i]:=true; nvar:=i;
              end;
          end;
        end;
      end;
      if nvar>0 then begin
        {analyse right side of the formula}
        j:=0;
        stk:=GetExpression(nvar,str,';',j);
        if stk=nil then inc(nerrors)
        else begin
          stk^.TrimToSize;
          stacks^[nvar]:=stk; inc(nform);
        end;
      end;
      if nerrprev<nerrors then begin
        AllExpressionsOK:=false;
        writeln(fout,'   ---⠫쭠     ( ";")');
        while not eof(fin) and (pos(';',str)<=0) do begin
          readln(fin,str); inc(nstr); str:=trim(str);
          writeln(fout,nstr,': ',str);
          writeln(fout,'   --- ');
        end;
      end;
    end;
  end;
  if nform=0 then begin
    inc(nerrors);
    writeln(fout,':  ⠭   !');
  end;
  close(fin); close(fout);
  {counting NVars and NConst}
  NVars:=0; NConst:=0;
  for i:=1 to nnames do
    if IsVariable^[i] and (stacks^[i]<>nil) then inc(NVars)
    else begin IsVariable^[i]:=false; inc(NConst); end;
  if (NVars<=0) or not AllExpressionsOK then begin
    {dispose all expressions}
    for i:=1 to nnames do begin
      if names^[i]<>nil then dispose(names^[i]);
      if stacks^[i]<>nil then dispose(stacks^[i],Done);
    end;
  end
  else begin
    GetMem(numbers,longint(nnames)*sizeof(integer));
    new(pds,Init); pds^.NVars:=NVars; pds^.NConst:=NConst;
    GetMem(pds^.VarNames,longint(NVars)*sizeof(PVarName));
    j:=0;
    for i:=1 to nnames do
      if IsVariable^[i] then begin
        inc(j); pds^.VarNames^[j]:=names^[i]; names^[i]:=nil;
        numbers^[i]:=j;
      end;
    if (NConst>0) then begin
      GetMem(pds^.ConstNames,longint(NConst)*sizeof(PVarName));
      j:=0;
      for i:=1 to nnames do
        if not IsVariable^[i] then begin
          inc(j); pds^.ConstNames^[j]:=names^[i]; names^[i]:=nil;
          numbers^[i]:=j;
        end;
    end;
    {change variable and constant numbers in expressions and
     rewrite them from the array stacks to the array Formulas}
    GetMem(pds^.Formulas,longint(NVars)*sizeof(PStack));
    k:=0;
    for i:=1 to nnames do
      if stacks^[i]<>nil then begin
        stk:=stacks^[i]; stacks^[i]:=nil;
        inc(k); pds^.Formulas^[k]:=stk;
        for j:=1 to stk^.size do begin
          pstel:=stk^.ElementAt(j);
          if (pstel^.ElType='c') or (pstel^.ElType='v') then begin
            if IsVariable^[pstel^.n] then pstel^.ElType:='v'
            else pstel^.ElType:='c';
            pstel^.n:=numbers^[pstel^.n];
          end;
        end;
      end;
  end;
  if numbers<>nil then FreeMem(numbers,longint(nnames)*sizeof(integer));
  dispose(names); dispose(IsVariable); dispose(stacks);
  parse:=pds;
end;

constructor TGetInitValues.Init;
begin
  IsVDefined:=nil; IsCDefined:=nil; pds:=nil;
end;

destructor TGetInitValues.Done;
begin
  if IsVDefined<>nil then
    FreeMem(IsVDefined,longint(pds^.NVars)*sizeof(boolean));
  if IsCDefined<>nil then
    FreeMem(IsCDefined,longint(pds^.NConst)*sizeof(boolean));
end;

procedure TGetInitValues.SetDynSystem(pDynSys:PDynSystem);
var i:integer;
begin
  pds:=pDynSys;
  if pds^.NVars>0 then begin
    GetMem(IsVDefined,longint(pds^.NVars)*sizeof(boolean));
    for i:=1 to pds^.NVars do IsVDefined^[i]:=false;
  end;
  if pds^.NConst>0 then begin
    GetMem(IsCDefined,longint(pds^.NConst)*sizeof(boolean));
    for i:=1 to pds^.NConst do IsCDefined^[i]:=false;
  end;
end;

function TGetInitValues.AreAllDefined:boolean;
var i:integer;
begin
  AreAllDefined:=false;
  if IsVDefined=nil then exit;
  for i:=1 to pds^.NVars do
    if not IsVDefined^[i] then exit;
  if IsCDefined<>nil then
    for i:=1 to pds^.NConst do
      if not IsCDefined^[i] then exit;
  AreAllDefined:=true;
end;

function TGetInitValues.ReadValueFromString(str:string):boolean;
var
  r:real;
  i,j:integer;
  k:longint;
  ident:string;
begin
  ReadValueFromString:=false;
  if pds=nil then exit;
  if str='' then exit;
  i:=pos('=',str);
  if i<=0 then exit;
  ident:=trim(copy(str,1,i-1));
  if not CheckId(ident) then exit;
  str:=trim(copy(str,i+1,255));
  if str='' then exit;
  if (ident='T') or (ident='N') then begin {a time variable}
    if not Str2Long(str,k) then exit;
    pds^.t0:=k
  end
  else begin
    if not Str2Real(str,r) then exit;
    {looking ident among variables}
    j:=1;
    while (j<=pds^.NVars) and (pds^.VarNames^[j]^<>ident) do inc(j);
    if j<=pds^.NVars then begin
      pds^.v0^[j]:=r; IsVDefined^[j]:=true;
    end
    else begin
      j:=1;
      while (j<=pds^.NConst) and (pds^.ConstNames^[j]^<>ident) do inc(j);
      if j>pds^.NVars then exit;
      pds^.c0^[j]:=r; IsCDefined^[j]:=true;
    end;
  end;
  ReadValueFromString:=true;
end;

function TGetInitValues.ReadValuesFromFile(var fin:text):boolean;
var
  OK:boolean;
  str:string;
begin
  OK:=true;
  while not eof(fin) do begin
    readln(fin,str);
    if str<>'' then OK:=ReadValueFromString(str) and OK;
  end;
end;

function TGetInitValues.ReadConditionFromString(str:string):PCondition;
{permissible formats: x [=,>,<,>=,<=] n; n [=,>,<,>=,<=] x;
 n1 [<,<=] x [<,<=] n2, where x-identifier, n,n1,n2-numbers}
{< is equal to <=; > is equal to >=}
var
  i,j,k:integer;
  r1,r2:real;
  l:longint;
  num1,num2:boolean;
  pc:PCondition;
  parts:array[1..5] of string;
  id:string;
  nparts:integer;
begin
  ReadConditionFromString:=nil;
  if str='' then exit;
  pc:=nil; nparts:=0;
  i:=pos('>',str);
  if i>0 then begin
    parts[1]:=trim(copy(str,1,i-1));
    parts[2]:='>';
    if str[i+1]='=' then inc(i);
    parts[3]:=trim(copy(str,i+1,255));
    nparts:=3;
  end;
  if nparts=0 then begin
    i:=pos('<',str);
    if i>0 then begin
      parts[1]:=trim(copy(str,1,i-1));
      parts[2]:='<';
      if str[i+1]='=' then inc(i);
      parts[3]:=trim(copy(str,i+1,255));
      nparts:=3;
      i:=pos('<',parts[3]);
      if (i>0) then begin
        parts[4]:='<';
        j:=i; if parts[3][i+1]='=' then inc(j);
        parts[5]:=trim(copy(parts[3],j+1,255));
        parts[3]:=trim(copy(parts[3],1,i-1));
        nparts:=5;
      end;
    end
  end;
  if nparts=0 then begin
    i:=pos('=',str);
    if i>0 then begin
      parts[1]:=trim(copy(str,1,i-1));
      parts[2]:='=';
      parts[3]:=trim(copy(str,i+1,255));
      nparts:=3;
    end;
  end;
  if nparts<3 then exit;
  for i:=1 to nparts do if parts[i]='' then exit;
  if nparts=5 then begin
    if not Str2Real(parts[1],r1) or not Str2Real(parts[5],r2) then exit;
    if not CheckId(parts[3]) then exit;
    i:=1;
    while (i<=pds^.NVars) and (pds^.VarNames^[i]^<>parts[3]) do inc(i);
    if i>pds^.NVars then exit;
    new(pc);
    pc^.VarN:=i;
    pc^.IsLowDefined:=true; pc^.IsHighDefined:=true;
    pc^.LowBound:=r1; pc^.HighBound:=r2;
  end
  else begin
    num1:=Str2Real(parts[1],r1); num2:=Str2Real(parts[3],r2);
    if num1=num2 then exit;
    if num1 then id:=parts[3] else id:=parts[1];
    if not CheckId(id) then exit;
    if (id='T') or (id='N') then begin {time variable}
      if num1 and (parts[2]='>') then exit;
      if num2 and (parts[2]='<') then exit; {only high bound is allowed}
      if num1 and not Str2Long(parts[1],l) then exit;
      if num2 and not Str2Long(parts[3],l) then exit;
      pds^.IsTDefined:=true; pds^.tfin:=l;
      exit;
    end;
    i:=1;
    while (i<=pds^.NVars) and (pds^.VarNames^[i]^<>id) do inc(i);
    if i>pds^.NVars then exit;
    new(pc);
    pc^.VarN:=i;
    pc^.IsLowDefined:=false; pc^.IsHighDefined:=false;
    if num1 then begin
      if (parts[2]='>') or (parts[2]='=') then begin
        pc^.IsHighDefined:=true; pc^.HighBound:=r1;
      end;
      if (parts[2]='<') or (parts[2]='=') then begin
        pc^.IsLowDefined:=true; pc^.LowBound:=r1;
      end;
    end
    else begin
      if (parts[2]='>') or (parts[2]='=') then begin
        pc^.IsLowDefined:=true; pc^.LowBound:=r2;
      end;
      if (parts[2]='<') or (parts[2]='=') then begin
        pc^.IsHighDefined:=true; pc^.HighBound:=r2;
      end;
    end;
  end;
  ReadConditionFromString:=pc;
end;

procedure TGetInitValues.ReadFinishConditionsFromFile(var fin:text);
var
  str:string;
  pc:PCondition;
  pclist:PCondArray;
  ncond,i:integer;
begin
  new(pclist); ncond:=0;
  while not eof(fin) do begin
    readln(fin,str);
    if str<>'' then begin
      pc:=ReadConditionFromString(str);
      if pc<>nil then begin
        inc(ncond); pclist^[ncond]:=pc;
      end;
    end;
  end;
  if ncond>0 then begin
    pds^.ncond:=ncond;
    GetMem(pds^.cond,longint(ncond)*sizeof(PCondition));
    for i:=1 to ncond do pds^.cond^[i]:=pclist^[i];
  end;
  dispose(pclist);
end;

end.


