{$A+,B-,D+,E+,F+,G-,I+,L+,N+,O+,P-,Q+,R+,S+,T-,V-,X+,Y+}
{$M 1024,0,0}

unit TraxPlot;

interface

uses WinTypes, WinProcs, OWindows, Strings,
     TpString,
     TraxRun;

const WM_NextPoint=WM_USER+1; WM_Reset=WM_USER+2; WM_Shut=WM_USER+10;
type
  TPlotWin=object(TWindow)
    pds:PDynSystem;
    nv1,nv2:integer;
    v1min,v1max,v2min,v2max:single;
    PrevH,PrevV:longint;
    dx,dy:integer;
    PointDrawn:boolean;
    IfMove:boolean;
    rc: PRect;

    DCMem:HDC;
    hbm, oldbm:HBITMAP;
    DCMemExist:boolean;

    constructor Init (AParent:PWindowsObject;
                      ThePDS:PDynSystem;
                      anv1,anv2:integer; {numbers of variables in dyn.system}
                      av1min,av1max,av2min,av2max:single);
                        {bounds for building graphics}
    destructor Done; virtual;
    function GetClassName:PChar; virtual;
    procedure GetWindowClass(var WndClass:TWndClass); virtual;
    function ScrX(r:single):longint; {from real number to screen coordinate}
    function ScrY(r:single):longint;
    procedure Paint (PaintDC: HDC; var PaintInfo: TPaintStruct); virtual;
    procedure DrawToDC (PaintDC: HDC);
    procedure NextPoint (var Msg:TMessage); virtual wm_first+WM_NextPoint;
    procedure Reset (var Msg:TMessage); virtual wm_first+WM_Reset;
    procedure Shut (var Msg:TMessage); virtual wm_first+WM_Shut;
    procedure WMRButtonDown (var Msg:TMessage); virtual wm_first+wm_RButtonDown;
    procedure WMRButtonUp (var Msg:TMessage); virtual wm_first+wm_RButtonUp;
    procedure WMMouseMove (var Msg:TMessage); virtual wm_first+wm_MouseMove;
    procedure DrawNext(DC:HDC);

    procedure MakeAltPage(DC:HDC);
    procedure DeleteAltPage;
    procedure MakeBitmap(DC:HDC);
    procedure DeleteBitmap;
    procedure CopyFromAltPage(DC:HDC);
  end;
  PPlotWin=^TPlotWin;

var PlotPens: array[1..7] of HPen;
const
  PlotColors:array[1..7,1..3] of byte=
    ((0,192,0),(192,0,0),(0,192,192),(0,0,192),
    (192,192,0),(192,164,0),(192,0,192));

implementation


constructor TPlotWin.Init (AParent:PWindowsObject;
                           ThePDS:PDynSystem;
                           anv1,anv2:integer;
                           av1min,av1max,av2min,av2max:single);
var p:array[0..100] of char;
begin
  inherited Init (AParent,StrPCopy(p,ThePDS^.VarNames^[anv2]^+'/'+ThePDS^.VarNames^[anv1]^));
  pds:=ThePDS;
  nv1:=anv1; nv2:=anv2;
  v1min:=av1min; v1max:=av1max; v2min:=av2min; v2max:=av2max;
  PrevH:=-1; PrevV:=-1; dx:=-1; dy:=-1;
  PointDrawn:=false;
  DCMem:=0; hbm:=0; oldbm:=0;
  DCMemExist:=false;
  IfMove:=FALSE;
  new(rc);
end;

destructor TPlotWin.Done;
begin
  dispose(rc);
  DeleteAltPage;
  inherited Done;
end;

procedure TPlotWin.MakeBitmap(DC:HDC);
begin
  if DCMem=0 then exit;
  if hbm<>0 then exit; {the bitmap already exists}
  hbm:=CreateCompatibleBitmap(DC,dx+20,dy+20);
  if hbm<>0 then
    if oldbm=0 then oldbm:=SelectObject(DCMem,hbm)
    else SelectObject(DCMem,hbm);
end;

procedure TPlotWin.MakeAltPage(DC:HDC);
var rect:TRect;
begin
  if DCMemExist then exit;
  DCMem:=CreateCompatibleDC(DC);
  DCMemExist:=true;
  MakeBitmap(DC);
  rect.left:=0; rect.top:=0; rect.right:=dx+21; rect.bottom:=dy+21;
  FillRect(DCMem,rect,GetStockObject(WHITE_BRUSH));
end;

procedure TPlotWin.DeleteBitmap;
begin
  if DCMemExist and (hbm<>0) then begin
    if oldbm<>0 then SelectObject(DCMem,oldbm);
    DeleteObject(hbm); hbm:=0;
  end;
end;

procedure TPlotWin.DeleteAltPage;
begin
  if DCMemExist then begin
    DeleteBitmap;
    DeleteDC(DCMem);
  end;
  DCMemExist:=false;
end;

procedure TPlotWin.CopyFromAltPage(DC:HDC);
begin
  if DCMemExist and (hbm<>0) then
    BitBlt(DC,0,0,dx+20,dy+20,DCMem,0,0,SRCCOPY);
end;

function TPlotWin.ScrX(r:single):longint;
begin
  ScrX:=10+round((r-v1min)/(v1max-v1min)*dx);
end;

function TPlotWin.ScrY(r:single):longint;
begin
  ScrY:=10+(dy-round((r-v2min)/(v2max-v2min)*dy));
end;

procedure TPlotWin.Paint (PaintDC: HDC; var PaintInfo: TPaintStruct);
var r:TRect;
begin
  if not DCMemExist then begin
    GetClientRect(HWindow,r);
    dx:=r.right-r.left-20;
    dy:=r.bottom-r.top-20;
    DrawToDC(PaintDC);
  end
  else
    CopyFromAltPage(PaintDC);
end;

procedure TPlotWin.DrawToDC (PaintDC: HDC);
var p,oldp:HPen;
    i,dh:integer;
    ps:array[0..50] of char;
    str:string;
    l:longint;
begin
  SetBKMode(PaintDC,TRANSPARENT);
  p:=CreatePen(PS_SOLID,1,RGB(128,128,128));
  oldp:=SelectObject(PaintDC,p);
  MoveTo(PaintDC,5,10); LineTo(PaintDC,10+dx+5,10);
  MoveTo(PaintDC,5,10+dy); LineTo(PaintDC,10+dx+5,10+dy);
  MoveTo(PaintDC,10,5); LineTo(PaintDC,10,10+dy+5);
  MoveTo(PaintDC,10+dx,5); LineTo(PaintDC,10+dx,10+dy+5);
  str:=trim(Real2Str(v2max,10,4));
  TextOut(PaintDC,0,0,StrPCopy(ps,str),length(str));
  str:=trim(Real2Str(v1max,10,4));
  l:=GetTextExtent(PaintDC,StrPCopy(ps,str),length(str));
  dh:=hiword(l);
  TextOut(PaintDC,20+dx-loword(l),20+dy-dh,ps,length(str));
  str:=trim(Real2Str(v1min,10,4));
  TextOut(PaintDC,10,20+dy-dh,StrPCopy(ps,str),length(str));
  str:=trim(Real2Str(v2min,10,4));
  TextOut(PaintDC,0,10+dy-dh,StrPCopy(ps,str),length(str));
  i:=ScrX(0.0);
  MoveTo(PaintDC,i,10+dy+5); LineTo(PaintDC,i,5);
  LineTo(PaintDC,i-3,5+3);
  MoveTo(PaintDC,i,5); LineTo(PaintDC,i+3,5+3);
  i:=ScrY(0.0);
  MoveTo(PaintDC,5,i); LineTo(PaintDC,10+dx+5,i);
  LineTo(PaintDC,10+dx+5-3,i-3);
  MoveTo(PaintDC,10+dx+5,i); LineTo(PaintDC,10+dx+5-3,i+3);
  SelectObject(PaintDC,oldp); DeleteObject(p);
end;

function TPlotWin.GetClassName:PChar;
begin
  GetClassname:='PlotWindow';
end;

procedure TPlotWin.GetWindowClass(var WndClass:TWndClass);
begin
  inherited GetWindowClass(WndClass);
  WndClass.hIcon:=LoadIcon(HInstance,'GraphWin');
end;

procedure TPlotWin.NextPoint (var Msg:TMessage);
var DC:HDC;
    oldbr:HBRUSH;
begin
  DC:=GetDC(HWindow);
  if not DCMemExist then begin
    MakeAltPage(DC);
    if DCMemExist then DrawToDC(DCMem);
  end;
  if DCMemExist then begin
    DrawNext(DCMem); CopyFromAltPage(DC);
  end
  else
    DrawNext(DC);
  if IfMove then begin
    oldbr:=SelectObject(DC,GetStockObject(NULL_BRUSH));
    Rectangle(DC, rc^.left, rc^.top, rc^.right, rc^.bottom);
    SelectObject(DC,oldbr);
  end;
  ReleaseDC(HWindow,DC);
end;

procedure TPlotWin.Reset (var Msg:TMessage);
var rect:TRect;
begin
  if DCMemExist then begin
    rect.left:=0; rect.top:=0; rect.right:=dx+21; rect.bottom:=dy+21;
    FillRect(DCMem,rect,GetStockObject(WHITE_BRUSH));
    DrawToDC(DCMem);
  end
  else begin
    InvalidateRect(HWindow,nil,true); UpdateWindow(HWindow);
  end;
  PointDrawn:=false;
  {SendMessage(HWindow,WM_NextPoint,0,0);}
end;

procedure TPlotWin.DrawNext(DC:HDC);
var x,y:single;
    NewH,NewV:integer;
    PrevUndefined:boolean;
    oldpen:HPen;
    k,t:longint;
    nc,np:integer;
begin
  if (pds^.nlast=0) or (pds^.last10[1]=nil) then begin
    {drawing one point}
    if nv1=0 then x:=pds^.tcurr
    else
      if not pds^.IsCurrUndefined^[nv1] then x:=pds^.VCurr^[nv1] else x:=0;
    if nv2=0 then y:=pds^.tcurr
    else
      if not pds^.IsCurrUndefined^[nv2] then y:=pds^.VCurr^[nv2] else y:=0;
    PrevUndefined:=((nv1>0) and pds^.IsPrevUndefined^[nv1]) or
                   ((nv2>0) and pds^.IsPrevUndefined^[nv2]);
    if pds^.ColorsChangeAfter>0 then begin
      k:=pds^.tcurr div pds^.ColorsChangeAfter+1;
      nc:=k mod pds^.NColors+1;
      oldpen:=SelectObject(DC,PlotPens[nc]);
    end;
    if PointDrawn and not PrevUndefined then begin
      NewH:=ScrX(x); NewV:=ScrY(y);
      MoveTo(DC,PrevH,PrevV);
      LineTo(DC,NewH,NewV);
      PrevH:=NewH; PrevV:=NewV;
    end
    else begin
      PrevH:=ScrX(x); PrevV:=ScrY(y);
      SetPixel(DC,PrevH,PrevV,0);
      PointDrawn:=true;
    end;
    if pds^.ColorsChangeAfter>0 then
      SelectObject(DC,oldpen);
  end
  else begin {drawing last 10 points}
    oldpen:=0;
    if PointDrawn and not PrevUndefined then MoveTo(DC,PrevH,PrevV);
    for np:=1 to pds^.nlast do begin
      t:=pds^.tcurr-pds^.nlast+np;
      if nv1=0 then x:=t
      else
        if not pds^.IsCurrUndefined^[nv1] then x:=pds^.last10[np]^[nv1] else x:=0;
      if nv2=0 then y:=t
      else
        if not pds^.IsCurrUndefined^[nv2] then y:=pds^.last10[np]^[nv2] else y:=0;
      PrevUndefined:=((nv1>0) and pds^.IsPrevUndefined^[nv1]) or
                     ((nv2>0) and pds^.IsPrevUndefined^[nv2]);
      if pds^.ColorsChangeAfter>0 then begin
        k:=t div pds^.ColorsChangeAfter+1;
        nc:=k mod pds^.NColors+1;
        if oldpen=0 then oldpen:=SelectObject(DC,PlotPens[nc])
        else SelectObject(DC,PlotPens[nc]);
      end;
      if PointDrawn and not PrevUndefined then begin
        NewH:=ScrX(x); NewV:=ScrY(y);
        LineTo(DC,NewH,NewV);
      end
      else begin
        PrevH:=ScrX(x); PrevV:=ScrY(y);
        SetPixel(DC,PrevH,PrevV,0); MoveTo(DC,PrevH,PrevV);
        PointDrawn:=true;
      end;
    end;
    if (pds^.ColorsChangeAfter>0) and (oldpen<>0) then
      SelectObject(DC,oldpen);
    PrevH:=NewH; PrevV:=NewV;
  end;
end;

procedure TPlotWin.Shut (var Msg:TMessage);
begin
  CloseWindow;
end;

procedure TPlotWin.WMRButtonDown(var Msg:TMessage);
begin
   if (not(IfMove)) then begin
      IfMove:=TRUE;
      SetCapture(HWindow);
      rc^.left:=Msg.LParamLo; rc^.top:=Msg.LParamHi;
      rc^.right:=rc^.left; rc^.bottom:=rc^.top;
   end;
end;

procedure TPlotWin.WMRButtonUp(var Msg:TMessage);
begin
    ReleaseCapture; IfMove:=false;
end;

procedure TPlotWin.WMMouseMove(var Msg:TMessage);
 var DC1: HDC;
     oldbr: HBRUSH;
 begin
    if (IfMove) then begin
     if (rc^.right<>Msg.LParamLo) or (rc^.bottom<>Msg.LParamHi) then begin
       InvalidateRect(HWindow, rc, True);
       UpdateWindow(HWindow);
     end;
     rc^.right:=Msg.LParamLo; rc^.bottom:=Msg.LParamHi;
     DC1:=GetDC(HWindow);
     oldbr:=SelectObject(DC1,GetStockObject(NULL_BRUSH));
     Rectangle(DC1, rc^.left, rc^.top, rc^.right, rc^.bottom);
     SelectObject(DC1,oldbr);
     ReleaseDC(HWindow,DC1);
   end;
 end;
end.